from django.apps import apps as django_apps

from django_injector.scope import RequestScope


def inject_request_middleware(get_response):
    app = django_apps.get_app_config('django_injector')

    def middleware(request):
        app.module.set_request(request)

        request_scope = app.injector.get(RequestScope)
        request_scope.prepare()

        try:
            return get_response(request)
        finally:
            request_scope.cleanup()

    return middleware



from functools import wraps
import inspect

from django_injector.scope import RequestScope, request_scope

from django.apps import apps as django_apps
from injector import inject as inject_


__version__ = '0.1.1'
__all__ = ['RequestScope', 'request_scope', 'inject']
default_app_config = 'django_injector.apps.DjangoInjectorConfig'


def inject(func):
    """Prepare a function or merhod for dependency injection.

    This decorator is used to mark functions and methods as injectable. The target
    can still receive non injected arguments. The order of arguments is important,
    all **non** injectable arguments should be listed first. All the injected arguments
    should have appropirate type annotations.

    >>> @inject
    >>> def my_view(request, some_instance: SomeClass, **kwargs):
    >>>     ...
    """
    signature = inspect.signature(func)
    is_method = 'self' in signature.parameters  # There must be a better way to do this

    func = inject_(func)

    @wraps(func)
    def wrapper(*args, **kwargs):
        injector = django_apps.get_app_config('django_injector').injector

        if is_method:
            self_, *args = args
        else:
            self_ = None

        return injector.call_with_injection(func, self_=self_, args=tuple(args), kwargs=kwargs)

    return wrapper



"""
Django settings for testproject project.

Generated by 'django-admin startproject' using Django 2.2.3.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '6scpl4djgk525$g6@pvpe+^76@bz1l&6v^%9&uf+_-pz5b8lm@'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['testserver']

# Application definition

INSTALLED_APPS = ['django_injector']

MIDDLEWARE = ['django_injector.middleware.DjangoInjectorMiddleware']

ROOT_URLCONF = 'testproject.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'testproject.wsgi.application'


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

INJECTOR_MODULES = ['testproject.testapp.modules.TestAppModule']



import random
from django.http import HttpRequest, HttpResponse
from django_injector import request_scope
from injector import inject


@inject
def inject_works(request, content: str):
    assert content == 'this is an injected string'
    return HttpResponse()


@request_scope
class RequestScopedService:
    def __init__(self):
        # We're using random numbers to make sure we can distinguish between services in
        # different scopes
        self.value = random.random()


@inject
def request_scope_works(request, service1: RequestScopedService, service2: RequestScopedService):
    # This makes sure that within a single request the scope will always return the same object
    assert service1 is service2

    # We return the id of the object we got in order for the code outside to verify that
    # objects change across requests
    return HttpResponse(str(service1.value))


class RequiresRequest:
    @inject
    def __init__(self, request: HttpRequest) -> None:
        self.request = request


@inject
def request_is_injectable(request, rr: RequiresRequest):
    assert rr.request is request
    return HttpResponse()



from inspect import isclass

from django.apps import AppConfig
from django.conf import settings
from django.utils.module_loading import import_string

from injector import Injector, Module

from .module import DjangoInjectorModule


class DjangoInjectorConfig(AppConfig):
    name = 'django_injector'

    def ready(self):
        super().ready()

        self.module = DjangoInjectorModule()
        modules = [self.module]

        for mod_str in getattr(settings, 'INJECTOR_MODULES', []):
            mod = import_string(mod_str)

            if isclass(mod):
                assert issubclass(mod, Module)
                mod = mod()

            modules.append(mod)

        self.injector = Injector(modules)



import threading

import injector


# The original code comes from the Flask-Injector project by Alec Thomas.
# Flask-Injector is licensed under the BSD 3-clause license.
class RequestScope(injector.Scope):
    """A scope whose object lifetime is tied to a request."""

    def cleanup(self) -> None:
        del self._locals.scope

    def prepare(self) -> None:
        self._locals.scope = {}

    def configure(self) -> None:
        self._locals = threading.local()

    def get(self, key, provider):
        try:
            return self._locals.scope[key]
        except KeyError:
            new_provider = injector.InstanceProvider(provider.get(self.injector))
            self._locals.scope[key] = new_provider
            return new_provider


request_scope = injector.ScopeDecorator(RequestScope)



import threading
import weakref

from django.conf import Settings, settings
from django.http import HttpRequest

from injector import Module, singleton


class DjangoInjectorModule(Module):

    def __init__(self):
        self._local = threading.local()

    def set_request(self, request):
        self._local.request = request

        if request:
            weakref.finalize(request, self.set_request, None)

    def configure(self, binder) -> None:
        binder.bind(Settings, to=settings, scope=singleton)
        binder.bind(HttpRequest, to=lambda: self._local.request)



import setuptools


with open('README.md') as fp:
    long_description = fp.read()


setuptools.setup(
    name='django_injector',
    version='0.1.1',
    author='Tiemo Kieft',
    author_email='pypi@isogram.nl',
    description='Integrate injector with Django',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/blubber/django_injector',
    packages=setuptools.find_packages(),
    install_requires=['injector'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
)



from django.urls import path

from . import views

urlpatterns = [
    path('inject-works', views.inject_works),
    path('request-scope-works', views.request_scope_works),
    path('request-is-injectable', views.request_is_injectable),
]
